package com.aliyun.iot.homelink.demo.page.scene.data;

import android.text.TextUtils;
import android.util.Log;

import com.alibaba.fastjson.JSON;
import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClient;
import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClientFactory;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTCallback;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTResponse;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequest;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequestBuilder;
import com.aliyun.iot.aep.sdk.threadpool.ThreadPool;
import com.aliyun.iot.homelink.demo.page.scene.api.BaseResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ConvertListener;
import com.aliyun.iot.homelink.demo.page.scene.api.DetailsResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.HousesResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.LogsResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ResponseCallback;
import com.aliyun.iot.homelink.demo.page.scene.api.SceneFireResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ScenesResponse;
import com.aliyun.iot.homelink.demo.page.scene.entities.Order;

import java.util.List;

/**
 * The type Scene repository.
 *
 * @author sinyuk
 * @date 2018 /6/14
 */
public class SceneRepository {
    private static final int SUCCEED_CODE = 200;
    private static final String TAG = "SceneRequest";

    private static class SingletonHolder {
        private static final SceneRepository INSTANCE = new SceneRepository();
    }

    private SceneRepository() {
    }

    /**
     * Gets instance.
     *
     * @return the instance
     */
    public static SceneRepository getInstance() {
        return SingletonHolder.INSTANCE;
    }

    /**
     * Get current house
     *
     * @param listener the listener
     */
    public void requestDefaultHouse(final ResponseCallback<HousesResponse.House> listener) {
        String apiVersion = "1.0.0";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/homelink/house/list")
                .setApiVersion(apiVersion)
                .addParam("pageNo", 1)
                .addParam("pageSize", 1);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    final HousesResponse data =
                            JSON.parseObject(response.getData().toString(), HousesResponse.class);
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            if (data.data != null && !data.data.isEmpty()) {
                                listener.onSuccess(data.data.get(0));
                            } else {
                                listener.onFail("没有房屋数据");
                            }
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }

    /**
     * Request scenes.
     *
     * @param pageNo   the page
     * @param pageSize the page size
     * @param groupId  the group id
     * @param listener the listener
     */
    public void requestScenes(final int pageNo,
                              final int pageSize,
                              final String groupId,
                              final ResponseCallback<ScenesResponse> listener) {
        String apiVersion = "1.0.1";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/scene/list/get")
                .setApiVersion(apiVersion)
                .addParam("pageNo", pageNo)
                .addParam("pageSize", pageSize)
                .addParam("groupId", groupId);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    final ScenesResponse data =
                            JSON.parseObject(response.getData().toString(), ScenesResponse.class);
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onSuccess(data);
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }

    /**
     * Reorder scenes.
     *
     * @param newOrders the new orders
     * @param groupId   the group id
     * @param listener  the listener
     */
    public void reorderScenes(final List<Order> newOrders,
                              final String groupId,
                              final ResponseCallback<BaseResponse> listener) {
        String apiVersion = "1.0.1";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/scene/list/reorder")
                .setApiVersion(apiVersion)
                .addParam("newOrders", JSON.parseArray(JSON.toJSONString(newOrders)))
                .addParam("groupId", groupId);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    final BaseResponse data =
                            JSON.parseObject(response.getData().toString(), BaseResponse.class);
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onSuccess(data);
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }


    /**
     * Request logs.
     *
     * @param pageNo   the page no
     * @param pageSize the page size
     * @param nowTime  the now time
     * @param groupId  the group id
     * @param listener the listener
     */
    public void requestLogs(final int pageNo,
                            final int pageSize,
                            final long nowTime,
                            final String groupId,
                            final ResponseCallback<LogsResponse> listener) {
        String apiVersion = "1.0.1";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/scene/log/list/get")
                .setApiVersion(apiVersion)
                .addParam("pageNo", pageNo)
                .addParam("pageSize", pageSize)
                .addParam("nowTime", nowTime)
                .addParam("groupId", groupId);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    final LogsResponse data =
                            JSON.parseObject(response.getData().toString(), LogsResponse.class);
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onSuccess(data);
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }

    /**
     * Request failed logs.
     *
     * @param logId    the log id
     * @param sceneId  the scene id
     * @param pageNo   the page no
     * @param pageSize the page size
     * @param listener the listener
     */
    public void requestFailedLogs(final String groupId,
                                  final String logId,
                                  final String sceneId,
                                  final int pageNo,
                                  final int pageSize,
                                  final long time,
                                  final ResponseCallback<DetailsResponse> listener) {
        String apiVersion = "1.0.3";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/scene/failedlog/get")
                .setApiVersion(apiVersion)
                .addParam("pageNo", pageNo)
                .addParam("pageSize", pageSize)
                .addParam("groupId", groupId)
                .addParam("time", time)
                .addParam("logId", logId)
                .addParam("sceneId", sceneId);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    final DetailsResponse data =
                            JSON.parseObject(response.getData().toString(), DetailsResponse.class);
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onSuccess(data);
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }

    /**
     * Fire scene.
     *
     * @param sceneId  the scene id
     * @param groupId  the group id
     * @param listener the listener
     */
    public void fireScene(String sceneId, String groupId, final ResponseCallback<String> listener) {
        String apiVersion = "1.0.1";
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/scene/fire")
                .setApiVersion(apiVersion)
                .addParam("groupId", groupId)
                .addParam("sceneId", sceneId);
        request(builder.build(), new ConvertListener() {
            @Override
            public void onError(final String message) {
                if (listener == null) {
                    return;
                }
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        listener.onFail(message);
                    }
                });
            }

            @Override
            public void onSucceed(final IoTResponse response) {
                if (listener == null) {
                    return;
                }
                try {
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onSuccess(response.getData().toString());
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                    final String error = e.getMessage();
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            listener.onFail(error);
                        }
                    });
                }
            }
        });
    }

    private void request(final IoTRequest request, final ConvertListener listener) {
        IoTAPIClient mIoTAPIClient = new IoTAPIClientFactory().getClient();
        mIoTAPIClient.send(request, new IoTCallback() {
            @Override
            public void onFailure(final IoTRequest ioTRequest, final Exception e) {
                if (e == null) {
                    return;
                }
                if (TextUtils.isEmpty(e.getLocalizedMessage())) {
                    listener.onError(e.getMessage());
                } else {
                    listener.onError(e.getLocalizedMessage());
                }
            }

            @Override
            public void onResponse(final IoTRequest ioTRequest, final IoTResponse ioTResponse) {
                if (SUCCEED_CODE != ioTResponse.getCode()) {
                    if (TextUtils.isEmpty(ioTResponse.getLocalizedMsg())) {
                        listener.onError(ioTResponse.getMessage());
                    } else {
                        listener.onError(ioTResponse.getLocalizedMsg());
                    }
                } else {
                    listener.onSucceed(ioTResponse);
                }
            }
        });
    }
}
