package com.aliyun.iot.homelink.demo.page.scene.list;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;

import com.aliyun.iot.homelink.demo.page.scene.api.BaseResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.HousesResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ResponseCallback;
import com.aliyun.iot.homelink.demo.page.scene.api.SceneFireResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ScenesResponse;
import com.aliyun.iot.homelink.demo.page.scene.data.SceneRepository;
import com.aliyun.iot.homelink.demo.page.scene.entities.Order;

import java.util.List;

import static com.aliyun.iot.homelink.demo.page.scene.Utils.checkNotNull;

/**
 * @author sinyuk
 * @date 2018/6/14
 */
public class SceneListPresenter implements SceneListContract.Presenter {
    public static final String TAG = "SceneListPresenter";
    private final SceneListContract.View sceneListView;
    private final SceneRepository sceneRepository;
    private String groupId;
    private int role;

    private static final int PAGE_SIZE = 10;

    public SceneListPresenter(@NonNull SceneRepository repository,
                              @NonNull SceneListContract.View view,
                              @Nullable String groupId,
                              int role) {
        this.groupId = groupId;
        this.role = role;
        sceneListView = view;
        sceneRepository = repository;
        sceneListView.setPresenter(this);
    }

    @Override
    public void start() {
        requestBackdropUrl();
        if (null == groupId) {
            loadDefaultHouse();
            // 在没有房屋的情况下
            sceneListView.updateRole(2);
        } else {
            loadScenes(1);
            sceneListView.updateRole(role);
        }
    }

    @Override
    public void requestBackdropUrl() {
        sceneListView.showBackdrop("");
    }

    @Override
    public void loadScenes(final int pageNo) {
        if (groupId == null) {
            loadDefaultHouse();
            return;
        }
        if (1 == pageNo) {
            sceneListView.showRefresh();
        }
        sceneRepository.requestScenes(pageNo, PAGE_SIZE, groupId, new ResponseCallback<ScenesResponse>() {
            @Override
            public void onFail(String message) {
                if (1 == pageNo) {
                    sceneListView.refreshFailed(message);
                } else {
                    sceneListView.loadMoreFailed(message);
                }
            }

            @Override
            public void onSuccess(ScenesResponse data) {
                if (1 == pageNo) {
                    sceneListView.hideRefresh();
                    sceneListView.initScenes(data);
                } else {
                    sceneListView.loadMoreScenes(data);
                }
            }
        });

    }

    @Override
    public void reorder(List<Order> orders) {
        checkNotNull(groupId);
        sceneRepository.reorderScenes(orders, groupId, new ResponseCallback<BaseResponse>() {
            @Override
            public void onFail(String message) {
                sceneListView.reorderFailed(message);
            }

            @Override
            public void onSuccess(BaseResponse data) {
                sceneListView.reorderSucceed();
            }
        });
    }

    @Override
    public void loadDefaultHouse() {
        sceneListView.showRefresh();
        sceneRepository.requestDefaultHouse(new ResponseCallback<HousesResponse.House>() {
            @Override
            public void onFail(String message) {
                sceneListView.hideRefresh();
                sceneListView.loadHouseFailed(message);
            }

            @Override
            public void onSuccess(HousesResponse.House data) {
                switchHouse(data.houseId, data.role);
            }
        });
    }

    @Override
    public void switchHouse(String houseId, int role) {
        setGroupId(houseId);
        setRole(role);
    }

    private void setRole(int role) {
        if (1 != role && 2 != role) {
            throw new IllegalArgumentException("role can't be: " + role);
        }
        if (role == this.role) {
            return;
        }
        this.role = role;
        if (sceneListView != null) {
            sceneListView.updateRole(role);
        }
    }

    private void setGroupId(String houseId) {
        checkNotNull(houseId);
        if (houseId.equals(groupId)) {
            return;
        }
        groupId = houseId;
        if (sceneListView != null) {
            loadScenes(1);
        }
    }

    @Override
    public void fire(final String sceneId) {
        checkNotNull(groupId);
        sceneRepository.fireScene(sceneId, groupId, new ResponseCallback<String>() {
            @Override
            public void onFail(String message) {
                sceneListView.onFireFailed(sceneId, message);
            }

            @Override
            public void onSuccess(String data) {
                sceneListView.onFireSucceed(sceneId);
            }
        });
    }

    @Override
    public String groupId() {
        return groupId;
    }

    @Override
    public int role() {
        return role;
    }
}
