package com.aliyun.iot.homelink.demo.page.scene.log;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Rect;
import android.support.v7.widget.RecyclerView;
import android.text.TextPaint;
import android.text.TextUtils;
import android.view.View;

/**
 * The type Scene log list decoration.
 *
 * @author sinyuk
 */
public class SceneLogListDecoration extends RecyclerView.ItemDecoration {
    private DecorationCallback callback;
    private TextPaint textPaint;
    private Paint paint;
    private int topGap;
    private int alignLeft;
    private Paint.FontMetrics fontMetrics;

    /**
     * Instantiates a new Scene log list decoration.
     *
     * @param context            the context
     * @param decorationCallback the decoration callback
     */
    SceneLogListDecoration(Context context, DecorationCallback decorationCallback) {
        this.callback = decorationCallback;
        //设置悬浮栏的画笔---paint
        paint = new Paint();
        paint.setColor(0xFFF6F6F6);

        //设置悬浮栏中文本的画笔
        textPaint = new TextPaint();
        textPaint.setAntiAlias(true);
        textPaint.setTextSize(dip2px(context, 12));
        textPaint.setColor(0xFF999999);
        textPaint.setTextAlign(Align.LEFT);
        fontMetrics = new Paint.FontMetrics();
        //决定悬浮栏的高度等
        topGap = dip2px(context, 36);
        alignLeft = dip2px(context, 16);
        //决定文本的显示位置等
        //alignBottom = 36;
    }

    private int dip2px(Context context, float dpValue) {
        float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }

    @Override
    public void getItemOffsets(Rect outRect, View view, RecyclerView parent, RecyclerView.State state) {
        super.getItemOffsets(outRect, view, parent, state);
        int pos = parent.getChildAdapterPosition(view);
        String timeStr = callback.getTimeStr(pos);
        if (TextUtils.isEmpty(timeStr)) {
            return;
        }
        //只有是同一组的第一个才显示悬浮栏
        if (callback.isFirstInGroup(pos)) {
            outRect.top = topGap;
        } else {
            outRect.top = 0;
        }
    }

    @Override
    public void onDraw(Canvas c, RecyclerView parent, RecyclerView.State state) {
        super.onDraw(c, parent, state);
        int left = parent.getPaddingLeft();
        int right = parent.getWidth() - parent.getPaddingRight();
        int childCount = parent.getChildCount();
        for (int i = 0; i < childCount; i++) {
            View view = parent.getChildAt(i);
            int position = parent.getChildAdapterPosition(view);
            String timeStr = callback.getTimeStr(position);
            if (TextUtils.isEmpty(timeStr)) {
                float top = view.getTop();
                float bottom = view.getTop();
                c.drawRect(left, top, right, bottom, paint);
                return;
            } else {
                if (position == 0 || callback.isFirstInGroup(position)) {
                    float top = view.getTop() - topGap;
                    float bottom = view.getTop();
                    //绘制悬浮栏
                    c.drawRect(left, top - topGap, right, bottom, paint);
                    //绘制文本
                    float baseline = (bottom + top - fontMetrics.bottom - fontMetrics.top) / 2 + textPaint.getTextSize() / 2;
                    c.drawText(timeStr, left + alignLeft, baseline, textPaint);
                }
            }
        }
    }

    @Override
    public void onDrawOver(Canvas c, RecyclerView parent, RecyclerView.State state) {
        super.onDrawOver(c, parent, state);
        int itemCount = state.getItemCount();
        int childCount = parent.getChildCount();
        int left = parent.getPaddingLeft();
        int right = parent.getWidth() - parent.getPaddingRight();
        String preTimeStr;
        String timeStr = "";
        for (int i = 0; i < childCount; i++) {
            View view = parent.getChildAt(i);
            int position = parent.getChildAdapterPosition(view);

            preTimeStr = timeStr;
            timeStr = callback.getTimeStr(position);
            if (TextUtils.isEmpty(timeStr) || timeStr.equals(preTimeStr)) {
                continue;
            }

            int viewBottom = view.getBottom();
            float textY = Math.max(topGap, view.getTop());
            //下一个和当前不一样移动当前
            if (position + 1 < itemCount) {
                String nextGroupId = callback.getTimeStr(position + 1);
                //组内最后一个view进入了header
                if (!nextGroupId.equals(timeStr) && viewBottom < textY) {
                    textY = viewBottom;
                }
            }
            //textY - topGap决定了悬浮栏绘制的高度和位置
            c.drawRect(left, textY - topGap, right, textY, paint);
            //left+2*alignBottom 决定了文本往左偏移的多少（加-->向左移）
            //textY-alignBottom  决定了文本往右偏移的多少  (减-->向上移)
            float baseline = (textY + textY - topGap - fontMetrics.bottom - fontMetrics.top) / 2 + textPaint.getTextSize() / 2;
            c.drawText(timeStr, left + alignLeft, baseline, textPaint);
        }
    }

    /**
     * The interface Decoration callback.
     */
    interface DecorationCallback {
        /**
         * Gets time str.
         *
         * @param position the position
         * @return the time str
         */
        String getTimeStr(int position);

        /**
         * Is first in group boolean.
         *
         * @param position the position
         * @return the boolean
         */
        boolean isFirstInGroup(int position);
    }
}